<?php

declare(strict_types=1);

namespace mirzaev\vk\arangodb\vk;

// Модуль ArangoDB для фреймворка ВКонтакте
use mirzaev\vk\arangodb\longpoll,
    mirzaev\vk\arangodb\journal;

// Фреймворк ArangoDB
use mirzaev\arangodb\collection,
    mirzaev\arangodb\document;

// Библиотека для работы с API ArabgoDB
use ArangoDBClient\Document as _document,
    ArangoDBClient\Connection as _connection;

/**
 * Чат ВКонтакте
 *
 * @package mirzaev\vk\arangodb\vk
 * @author Arsen Mirzaev Tatyano-Muradovich <arsen@mirzaev.sexy>
 */
class chat
{
    /**
     * Инициализация
     *
     * @param _connection $session Сессия соединения с базой данных
     * @param int $id Идентификатор ВКонтакте
     * @param bool $journal Записывать в журнал?
     * @param bool $create Создавать коллекции при их отсутствии?
     *
     * @return ?_document Инстанция документа
     */
    public static function init(_connection $session, int $id, bool $journal = true, bool $create = true): ?_document
    {
        // Инициализация коллекции
        $collection = longpoll::COLLECTION_CHATS;

        if ($create) {
            // Запрошено создание коллекций в случае их отсутствия

            // Создание коллекции
            collection::init($session, $collection);
        }

        if ($account = collection::search($session, <<<AQL
                FOR x IN $collection
                FILTER x.id == "$id"
                LIMIT 1
                RETURN x
            AQL)) {
            // Найден аккаунт

            return $account;
        }

        if ($account = document::write($session, $collection, ['id' => $id])) {
            // Аккаунт записан

            // Журналирование
            if ($journal && journal::init($session, $account)->write('create', [
                'changes' => [
                    'new' => collection::search($session, <<<AQL
                            FOR x IN $collection
                            FILTER x._id == "$account"
                            LIMIT 1
                            RETURN x
                        AQL),
                    'old' => null
                ]
            ])) {
                // Записаны данные в журнал
            }

            if ($account = collection::search($session, <<<AQL
                    FOR x IN $collection
                    FILTER x.id == "$id"
                    LIMIT 1
                    RETURN x
                AQL)) {
                // Найден аккаунт

                return $account;
            }
        }

        return null;
    }
}
